@@ -25,11 +25,13 @@ musllinux_1_1_aarch64"
 PyVers="38
 39
 310
-311"
+311
+312
+"
 
 for os in $OSes; do
     for pyver in $PyVers; do
-        if [[ ${os} == "macosx_11_0_x86_64" && ${pyver} == "311" ]]; then
+        if [[ ${os} == "macosx_11_0_x86_64" && ${pyver} == "311" || ${os} == "macosx_11_0_x86_64" && ${pyver} == "312" ]]; then
             wget -q --directory-prefix=dist/ https://github.com/fastavro/fastavro/releases/download/${ver}/fastavro-${ver}-cp${pyver}-cp${pyver}-macosx_10_9_universal2.whl
         else
             wget -q --directory-prefix=dist/ https://github.com/fastavro/fastavro/releases/download/${ver}/fastavro-${ver}-cp${pyver}-cp${pyver}-${os}.whl
