@@ -6,14 +6,20 @@ import pytest
 
 from fastavro import json_writer, json_reader
 from fastavro.schema import parse_schema
+from fastavro.validation import ValidationError
+from fastavro.io.json_decoder import AvroJSONDecoder
+from fastavro.io.json_encoder import AvroJSONEncoder
+from fastavro.io.symbols import MapKeyMarker, String
 
 
-def roundtrip(schema, records):
+def roundtrip(schema, records, *, reader_schema=None, writer_kwargs={}):
     new_file = StringIO()
-    json_writer(new_file, schema, records)
+    json_writer(new_file, schema, records, **writer_kwargs)
     new_file.seek(0)
 
-    new_records = list(json_reader(new_file, schema))
+    reader = json_reader(new_file, schema, reader_schema)
+
+    new_records = list(reader)
     return new_records
 
 
@@ -129,27 +135,12 @@ def test_more_than_one_record():
         "name": "test_more_than_one_record",
         "namespace": "test",
         "fields": [
-            {
-                "name": "string",
-                "type": "string",
-            },
-            {
-                "name": "int",
-                "type": "int",
-            },
+            {"name": "string", "type": "string"},
+            {"name": "int", "type": "int"},
         ],
     }
 
-    records = [
-        {
-            "string": "foo",
-            "int": 1,
-        },
-        {
-            "string": "bar",
-            "int": 2,
-        },
-    ]
+    records = [{"string": "foo", "int": 1}, {"string": "bar", "int": 2}]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -169,12 +160,7 @@ def test_encoded_union_output():
                     {
                         "type": "record",
                         "name": "union_record",
-                        "fields": [
-                            {
-                                "name": "union_record_field",
-                                "type": "string",
-                            }
-                        ],
+                        "fields": [{"name": "union_record_field", "type": "string"}],
                     },
                 ],
             }
@@ -204,7 +190,8 @@ def test_encoded_union_output():
     assert new_file.getvalue().strip() == expected
 
 
-def test_union_string_and_bytes():
+def test_union_output_without_type():
+    """https://github.com/fastavro/fastavro/issues/420"""
     schema = {
         "type": "record",
         "name": "Test",
@@ -213,19 +200,47 @@ def test_union_string_and_bytes():
             {
                 "name": "union",
                 "type": [
-                    "string",
-                    "bytes",
+                    "null",
+                    "int",
+                    {
+                        "type": "record",
+                        "name": "union_record",
+                        "fields": [{"name": "union_record_field", "type": "string"}],
+                    },
                 ],
             }
         ],
     }
 
-    records = [
-        {
-            "union": "asdf",
-        },
-        {"union": b"asdf"},
-    ]
+    # A null value is encoded as just null
+    records = [{"union": None}]
+    new_file = StringIO()
+    json_writer(new_file, schema, records, write_union_type=False)
+    assert new_file.getvalue().strip() == json.dumps({"union": None})
+
+    # A non-null, non-named type is encoded as just the value
+    records = [{"union": 321}]
+    new_file = StringIO()
+    json_writer(new_file, schema, records, write_union_type=False)
+    assert new_file.getvalue().strip() == json.dumps({"union": 321})
+
+    # A non-null, named type is encoded as an object
+    records = [{"union": {"union_record_field": "union_field"}}]
+    new_file = StringIO()
+    json_writer(new_file, schema, records, write_union_type=False)
+    expected = json.dumps({"union": {"union_record_field": "union_field"}})
+    assert new_file.getvalue().strip() == expected
+
+
+def test_union_string_and_bytes():
+    schema = {
+        "type": "record",
+        "name": "Test",
+        "namespace": "test",
+        "fields": [{"name": "union", "type": ["string", "bytes"]}],
+    }
+
+    records = [{"union": "asdf"}, {"union": b"asdf"}]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -243,10 +258,7 @@ def test_simple_type():
 def test_array_type_simple():
     schema = {"type": "array", "items": "string"}
 
-    records = [
-        ["foo", "bar"],
-        ["a", "b"],
-    ]
+    records = [["foo", "bar"], ["a", "b"]]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -259,22 +271,13 @@ def test_array_type_records():
             "type": "record",
             "name": "test_array_type",
             "fields": [
-                {
-                    "name": "field1",
-                    "type": "string",
-                },
-                {
-                    "name": "field2",
-                    "type": "int",
-                },
+                {"name": "field1", "type": "string"},
+                {"name": "field2", "type": "int"},
             ],
         },
     }
 
-    records = [
-        [{"field1": "foo", "field2": 1}],
-        [{"field1": "bar", "field2": 2}],
-    ]
+    records = [[{"field1": "foo", "field2": 1}], [{"field1": "bar", "field2": 2}]]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -282,15 +285,9 @@ def test_array_type_records():
 
 def test_empty_maps():
     """https://github.com/fastavro/fastavro/issues/380"""
-    schema = {
-        "type": "map",
-        "values": "int",
-    }
+    schema = {"type": "map", "values": "int"}
 
-    records = [
-        {"foo": 1},
-        {},
-    ]
+    records = [{"foo": 1}, {}]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -298,15 +295,9 @@ def test_empty_maps():
 
 def test_empty_arrays():
     """https://github.com/fastavro/fastavro/issues/380"""
-    schema = {
-        "type": "array",
-        "items": "int",
-    }
+    schema = {"type": "array", "items": "int"}
 
-    records = [
-        [1],
-        [],
-    ]
+    records = [[1], []]
 
     new_records = roundtrip(schema, records)
     assert records == new_records
@@ -320,22 +311,12 @@ def test_union_in_array():
             {
                 "type": "record",
                 "name": "rec1",
-                "fields": [
-                    {
-                        "name": "field1",
-                        "type": ["string", "null"],
-                    }
-                ],
+                "fields": [{"name": "field1", "type": ["string", "null"]}],
             },
             {
                 "type": "record",
                 "name": "rec2",
-                "fields": [
-                    {
-                        "name": "field2",
-                        "type": ["string", "null"],
-                    }
-                ],
+                "fields": [{"name": "field2", "type": ["string", "null"]}],
             },
             "null",
         ],
@@ -405,10 +386,7 @@ def test_union_in_map():
         "fields": [
             {
                 "name": "map",
-                "type": {
-                    "type": "map",
-                    "values": ["string", "null"],
-                },
+                "type": {"type": "map", "values": ["string", "null"]},
             }
         ],
     }
@@ -455,16 +433,9 @@ def test_enum_named_type():
         "fields": [
             {
                 "name": "test1",
-                "type": {
-                    "type": "enum",
-                    "name": "my_enum",
-                    "symbols": ["FOO", "BAR"],
-                },
-            },
-            {
-                "name": "test2",
-                "type": "my_enum",
+                "type": {"type": "enum", "name": "my_enum", "symbols": ["FOO", "BAR"]},
             },
+            {"name": "test2", "type": "my_enum"},
         ],
     }
 
@@ -481,16 +452,9 @@ def test_fixed_named_type():
         "fields": [
             {
                 "name": "test1",
-                "type": {
-                    "type": "fixed",
-                    "name": "my_fixed",
-                    "size": 4,
-                },
-            },
-            {
-                "name": "test2",
-                "type": "my_fixed",
+                "type": {"type": "fixed", "name": "my_fixed", "size": 4},
             },
+            {"name": "test2", "type": "my_fixed"},
         ],
     }
 
@@ -510,18 +474,10 @@ def test_record_named_type():
                 "type": {
                     "type": "record",
                     "name": "my_record",
-                    "fields": [
-                        {
-                            "name": "field1",
-                            "type": "string",
-                        }
-                    ],
+                    "fields": [{"name": "field1", "type": "string"}],
                 },
             },
-            {
-                "name": "test2",
-                "type": "my_record",
-            },
+            {"name": "test2", "type": "my_record"},
         ],
     }
 
@@ -685,3 +641,264 @@ def test_default_value_missing():
     new_file = StringIO(json.dumps(record))
     with pytest.raises(ValueError, match="no value and no default"):
         next(json_reader(new_file, schema))
+
+
+def test_map_of_union_of_array_and_map():
+    """https://github.com/fastavro/fastavro/issues/572"""
+    schema = {
+        "name": "Test",
+        "namespace": "test",
+        "type": "record",
+        "fields": [
+            {
+                "name": "metadata",
+                "type": {
+                    "type": "map",
+                    "values": [
+                        {"type": "array", "items": "string"},
+                        {"type": "map", "values": ["string"]},
+                    ],
+                },
+            }
+        ],
+    }
+
+    records = [{"metadata": {"map1": {"map2": "str"}}}]
+
+    new_records = roundtrip(schema, records)
+    assert records == new_records
+
+
+def test_json_writer_with_validation():
+    """https://github.com/fastavro/fastavro/issues/580"""
+    schema = {
+        "doc": "A weather reading.",
+        "name": "Weather",
+        "namespace": "test",
+        "type": "record",
+        "fields": [
+            {"name": "station", "type": "string"},
+            {"name": "time", "type": "long"},
+            {"name": "temp", "type": "int"},
+        ],
+    }
+
+    records = [
+        {"station": "011990-99999", "temp": 0, "time": 1433269388},
+        {"station": "011990-99999", "temp": 22, "time": "last day"},
+        {"station": "011990-99999", "temp": -11, "time": 1433273379},
+        {"station": "012650-99999", "temp": 111.9, "time": 1433275478},
+    ]
+
+    new_file = StringIO()
+    with pytest.raises(ValidationError):
+        json_writer(new_file, schema, records, validator=True)
+
+
+def test_custom_encoder_and_decoder():
+    """https://github.com/fastavro/fastavro/pull/579"""
+
+    class CustomJSONEncoder(AvroJSONEncoder):
+        """Encoder that will prepend an underscore to all string values"""
+
+        def write_utf8(self, value):
+            self._parser.advance(String())
+            if self._parser.stack[-1] == MapKeyMarker():
+                self._parser.advance(MapKeyMarker())
+                self.write_object_key(value)
+            else:
+                self.write_value("_" + value)
+
+    class CustomJSONDecoder(AvroJSONDecoder):
+        """Decoder that will prepend an underscore to all string values"""
+
+        def read_utf8(self, **kwargs):
+            symbol = self._parser.advance(String())
+            if self._parser.stack[-1] == MapKeyMarker():
+                self._parser.advance(MapKeyMarker())
+                for key in self._current:
+                    self._key = key
+                    break
+                return self._key
+            else:
+                return "_" + self.read_value(symbol)
+
+    schema = {
+        "doc": "A weather reading.",
+        "name": "Weather",
+        "namespace": "test",
+        "type": "record",
+        "fields": [
+            {"name": "station", "type": "string"},
+            {"name": "time", "type": "long"},
+            {"name": "temp", "type": "int"},
+        ],
+    }
+
+    records = [
+        {"station": "011990-99999", "temp": 0, "time": 1433269388},
+        {"station": "011990-99999", "temp": 22, "time": 1433269389},
+        {"station": "011990-99999", "temp": -11, "time": 1433273379},
+        {"station": "012650-99999", "temp": 111.9, "time": 1433275478},
+    ]
+
+    new_file = StringIO()
+    json_writer(new_file, schema, records, encoder=CustomJSONEncoder)
+    assert "_011990-99999" in new_file.getvalue()
+
+    new_file.seek(0)
+    for record in json_reader(new_file, schema, decoder=CustomJSONDecoder):
+        assert record["station"].startswith("__")
+
+
+def test_strict_option():
+    """https://github.com/fastavro/fastavro/issues/549"""
+    schema = {
+        "namespace": "namespace",
+        "name": "name",
+        "type": "record",
+        "fields": [
+            {"name": "field_1", "type": "boolean"},
+            {"name": "field_2", "type": ["null", "string"], "default": None},
+        ],
+    }
+
+    test_record1 = {"field_1": True, "field_2": "foo", "field_3": "something"}
+    test_record2 = {"field_1": True}
+    test_record3 = {"field_2": "foo"}
+
+    with pytest.raises(ValueError, match="field_3"):
+        roundtrip(schema, [test_record1], writer_kwargs={"strict": True})
+
+    with pytest.raises(ValueError, match="field_2 is specified .*? but missing"):
+        roundtrip(schema, [test_record2], writer_kwargs={"strict": True})
+
+    with pytest.raises(ValueError, match="field_1 is specified .*? but missing"):
+        roundtrip(schema, [test_record3], writer_kwargs={"strict": True})
+
+
+def test_strict_allow_default_option():
+    """https://github.com/fastavro/fastavro/issues/549"""
+    schema = {
+        "namespace": "namespace",
+        "name": "name",
+        "type": "record",
+        "fields": [
+            {"name": "field_1", "type": "boolean"},
+            {"name": "field_2", "type": ["null", "string"], "default": None},
+        ],
+    }
+
+    test_record1 = {"field_1": True, "field_2": "foo", "field_3": "something"}
+    test_record2 = {"field_1": True}
+    test_record3 = {"field_2": "foo"}
+
+    with pytest.raises(ValueError, match="field_3"):
+        roundtrip(schema, [test_record1], writer_kwargs={"strict_allow_default": True})
+
+    roundtrip(schema, [test_record2], writer_kwargs={"strict_allow_default": True})
+
+    with pytest.raises(ValueError, match="field_1 is specified .*? but missing"):
+        roundtrip(schema, [test_record3], writer_kwargs={"strict_allow_default": True})
+
+
+def test_json_with_map():
+    """https://github.com/fastavro/fastavro/issues/629"""
+    schema = {
+        "type": "record",
+        "name": "User",
+        "fields": [
+            {"name": "name", "type": "string"},
+            {"name": "age", "type": "long"},
+            {
+                "name": "addresses",
+                "type": {
+                    "type": "map",
+                    "values": {
+                        "type": "record",
+                        "name": "Address",
+                        "fields": [
+                            {"name": "street", "type": "string"},
+                            {"name": "street_number", "type": "long"},
+                        ],
+                        "doc": "An Address",
+                    },
+                    "name": "address",
+                },
+            },
+        ],
+        "doc": "User with multiple Address",
+    }
+
+    payload = {
+        "name": "TogYzVenzFrgcVunpkfo",
+        "age": 5694,
+        "addresses": {
+            "HGbZkCCabEbwaTXjbTEA": {
+                "street": "tNXiLgAYswaCPLazSfus",
+                "street_number": 1316,
+            }
+        },
+    }
+
+    roundtrip(schema, [payload])
+
+
+def test_disable_tuple_notation_option():
+    """https://github.com/fastavro/fastavro/issues/548"""
+    schema = {
+        "namespace": "namespace",
+        "name": "name",
+        "type": "record",
+        "fields": [
+            {"name": "foo", "type": ["string", {"type": "array", "items": "string"}]}
+        ],
+    }
+
+    new_records = roundtrip(
+        schema,
+        [{"foo": ("string", "0")}],
+        writer_kwargs={"disable_tuple_notation": True},
+    )
+    assert new_records == [{"foo": ["string", "0"]}]
+
+
+def test_strict_allow_default_bug():
+    """https://github.com/fastavro/fastavro/issues/638"""
+    schema = {
+        "namespace": "namespace",
+        "name": "name",
+        "type": "record",
+        "fields": [{"name": "some_field", "type": "string", "default": "test"}],
+    }
+
+    test_record = {"eggs": "eggs"}
+
+    with pytest.raises(ValueError, match="record contains more fields .*? eggs"):
+        roundtrip(schema, [test_record], writer_kwargs={"strict_allow_default": True})
+
+
+def test_json_aliases():
+    """https://github.com/fastavro/fastavro/issues/669"""
+    schema = {
+        "name": "test_json_aliases",
+        "type": "record",
+        "fields": [
+            {"name": "field_1", "type": "string"},
+            {"name": "field_2", "type": "long"},
+        ],
+    }
+
+    reader_schema = {
+        "name": "test_json_aliases",
+        "type": "record",
+        "fields": [
+            {"name": "new_field_1", "type": "string", "aliases": ["field_1"]},
+            {"name": "field_2", "type": "long"},
+        ],
+    }
+
+    records = [{"field_1": "foo", "field_2": 10}]
+
+    output_records = roundtrip(schema, records, reader_schema=reader_schema)
+    assert output_records == [{"new_field_1": "foo", "field_2": 10}]
